import { useState, useEffect } from "react";
import type { RegistryItem } from "../lib/types";

const RegistryList = () => {
	const [registryItems, setRegistryItems] = useState<RegistryItem[]>([]);
	const [claimedItems, setClaimedItems] = useState<Set<string>>(new Set());
	const [loading, setLoading] = useState(true);
	const [error, setError] = useState<string | null>(null);

	useEffect(() => {
		const fetchRegistryItems = async () => {
			setLoading(true);
			setError(null);
			try {
				const response = await fetch("/api/registry");
				if (!response.ok) {
					throw new Error(`HTTP error! status: ${response.status}`);
				}
				const data = await response.json();
				setRegistryItems(data);
			} catch (e: any) {
				setError(e.message);
				console.error("Failed to fetch registry items:", e);
			} finally {
				setLoading(false);
			}
		};

		fetchRegistryItems();
	}, []);

	const handleCheckboxChange = (itemId: string) => {
		const newClaimedItems = new Set(claimedItems);
		if (newClaimedItems.has(itemId)) {
			newClaimedItems.delete(itemId);
		} else {
			newClaimedItems.add(itemId);
		}
		setClaimedItems(newClaimedItems);
	};

	const handleSubmit = async () => {
		// Prepare updates for claimed items
		const updates = registryItems.filter((item) =>
			claimedItems.has(item.id)
		);

		// Optimistically update the UI
		const updatedRegistryItems = registryItems.map((item) => {
			if (claimedItems.has(item.id)) {
				return { ...item, taken: true };
			}
			return item;
		});
		setRegistryItems(updatedRegistryItems);
		setClaimedItems(new Set()); // Clear claimed items after submission

		try {
			// Send updates to the server
			for (const item of updates) {
				const response = await fetch(`/api/registry/${item.id}`, {
					method: "PUT",
					headers: {
						"Content-Type": "application/json",
					},
					body: JSON.stringify({ taken: true }),
				});

				if (!response.ok) {
					throw new Error(
						`Failed to update item ${item.id}: ${response.statusText}`
					);
				}
			}
			alert("Thank you for claiming items!");
		} catch (error: any) {
			console.error("Error updating items:", error);
			setError("Failed to update items. Please try again.");
			setRegistryItems(registryItems);
		}
	};

	if (loading) {
		return <div className="text-center">Loading registry items...</div>;
	}

	if (error) {
		return (
			<div className="text-red-500 text-center">Error: {error}</div>
		);
	}

	return (
		<div className="container mx-auto p-4">
			<h1 className="text-2xl font-bold mb-4">Baby Registry</h1>
			<div className="overflow-x-auto">
				<table className="table w-full">
					<thead>
						<tr>
							<th>Item</th>
							<th>Link</th>
							<th>Claim</th>
						</tr>
					</thead>
					<tbody>
						{registryItems.map((item) => (
							<tr key={item.id} className={item.taken ? "opacity-50" : ""}>
								<td>
									{item.name}
									{item.taken && (
										<span className="badge badge-success ml-2">Taken</span>
									)}
								</td>
								<td>
									{item.link && (
										<a
											href={item.link}
											target="_blank"
											rel="noopener noreferrer"
											className="link link-primary"
										>
											View
										</a>
									)}
								</td>
								<td>
									{!item.taken && (
										<input
											type="checkbox"
											className="checkbox"
											checked={claimedItems.has(item.id)}
											onChange={() => handleCheckboxChange(item.id)}
										/>
									)}
								</td>
							</tr>
						))}
					</tbody>
				</table>
			</div>
			<button
				className="btn btn-primary mt-4"
				onClick={handleSubmit}
				disabled={claimedItems.size === 0}
			>
				Submit Claims
			</button>
		</div>
	);
};

export default RegistryList;
